/*
 *  Program     :  SCTP SERVER ONE TO ONE
 *  Author      :  Andres Medina 
 *  Last review :  04/04/2012 
 *  Summary     :
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/sctp.h>
#include <errno.h>
#include <sys/types.h>

#define PORT 12345
#define BUFF_SIZE (1 << 16)

/*
 *           FUNCION MAIN
 */
int main(int argc, char * argv[]){

    int flags, data, sd, sock, ret, i, addr_count;
    struct sockaddr_in addr, paddr;
    struct sockaddr_in *laddr[10];
    socklen_t from_len, client_len;
    char buffer[BUFF_SIZE];
    struct sctp_paddrparams heartbeat;

    /* Se setean los valores de familia, puerto y direccion IP del socket */
    addr.sin_addr.s_addr = htons(INADDR_ANY);
    addr.sin_port = htons(PORT);
    addr.sin_family = AF_INET; 

    /* Se crea el socket */
    if ( (sd = socket(AF_INET, SOCK_STREAM, IPPROTO_SCTP )) < 0 ){
        printf("Error: cannot create socket\n");
        exit(-1);
    }

    from_len = sizeof(addr);

    /* Asociamos el socket a la direccion IP */
    if ( bind(sd, (struct sockaddr*) &addr, from_len) < 0){
        printf("Error: cannot bind the socket to the given IP address\n");
        exit(-1);
    }

    /* Se imprimen las IP local asociadas al socket */
    addr_count = sctp_getladdrs(sd, 0, (struct sockaddr **) laddr);
    printf("Addresses binded: %d\n", addr_count);
    for(i = 0; i < addr_count; i++)
         printf("Address %d: %s:%d\n", i +1, inet_ntoa((*laddr)[i].sin_addr), ntohs((*laddr)[i].sin_port));
    sctp_freeladdrs((struct sockaddr*)*laddr);


    /* Escuchamos el puerto para recibir conexiones */
    if ( (listen(sd, 1)) < 0 ){
        printf("Error: listen failed\n");
        exit(-1);
    }

    printf("Waiting for conections ...\n");

    /* Se reciben los mensajes enviados por el cliente */
    while(1){

        /* Aceptamos la conexion en caso de detectar alguna */
        sock = accept(sd, (struct sockaddr*) &addr, &from_len);

        flags = 0;      

        while ((ret = recv(sock, buffer, BUFF_SIZE, flags)) > 0){
       
           if ( getsockname(sock, (struct sockaddr*)&addr, &from_len) < 0){
               printf("Error: getsockname failed\n");
               exit(-1);
           }

           printf("%d bytes received in Local Address: %s:%d.", ret, inet_ntoa(addr.sin_addr), ntohs(addr.sin_port));
 
           if ( getpeername(sock, (struct sockaddr*)&addr, &from_len) < 0){
               printf("Error: getpeername failed\n");
               exit(-1);
           }
 
           printf(" Data sent by Client: %s:%d\n", inet_ntoa(addr.sin_addr), ntohs(addr.sin_port));
        
        }

    }   


    /* Cerramos el socket en caso de terminar el programa */
    if( close(sd) < 0 ){
        printf("Error: cannot close the socket file descriptor\n");
        exit(-1);
    }

    return 0;
}
